/*=============================================================================
  Copyright (c) YOUR_COPYRIGHT_MESSAGE_HERE

  File: MoveTowards_entry_action.h

  Author: YOU
  Date: DATE

  Description: { fill in with your module's ACCESS_ACTION handler, some common AN's are provided }

=============================================================================*/
#include "ActionExamples.h"


FX_ENTRYFUNC(MoveTowards_entry_action)
// FXint f(FX_AccessInfo *ai, FXentity ID, FXint64 entry)
{
	FXaction	act = (FXaction)ID;		// object that triggered this notification
	FXhandle	hand = FX_NULLID;			// handle this action is attached to
	FXobject	ctlr = FX_NULLID;			// controller (if any) for the handle


	hand = fxActionHandle( act, FX_NOFLAG );
	ctlr = fxHandleController( hand, FX_NOFLAG );

	switch( entry )
	{
	case ACT_PROCESS:
		{

			FXobject	target = FX_NULLID, lastTarget = FX_NULLID;
			FXint		num_selected = 0, i = 0;
			FXvecd		targetPos, ctlrPos, tmpVec, dir, mouse, plane;
			FXmatrix	wrldRot, invWrldRot;
			FXdouble	len, dp;
			FXint		keyID =	-1, keyCreated = 0, keyInclusion = 0;
			FXchar		mode[64];
			FXchar		itemName[64];

			num_selected = fxObjectNumSelected(FX_NOFLAG);
			num_selected = FX_MAX( num_selected, 1 );

			fxChannelsGet( ctlr, FX_WORLD_POSITION, NOW, ctlrPos, FX_NOFLAG );

			// cycle through all selected objects, summing their offsets in local space
			for( i = 0, FX_V3CLEAR(dir); i < num_selected; i++ )
			{
				if( i == 0 )	// first time through loop, get first selected
					target = fxObjectFirstSelected(FX_NOFLAG);
				else			// after that get next selected
					target = fxObjectNextSelected( lastTarget, FX_NOFLAG );

				if( target != ctlr ) // we don't want the controller to move towards itself!
				{
					// get the inverse rotation matrix of the controller
					fxMatrixGet( ctlr, FX_MATRIX_WORLD_FILTERED, NOW, wrldRot, 0 );
					_fxMatrixInvert( invWrldRot, wrldRot );

					// get the world position of the target
					fxChannelsGet( target, FX_WORLD_POSITION, NOW, targetPos, FX_NOFLAG );

					// translate the target by the inverse of the controller's world position
					FX_V3SUB( tmpVec, targetPos, ctlrPos );
					// rotate the target point by the inverse of the controller's rotation matrix
					_fxMatrixXform( targetPos, tmpVec, invWrldRot, 0 );

					// add the local space offset to the total offset of all targets
					FX_V3ADD1( dir, targetPos );
				}

				lastTarget = target;
			}

			// normalize direction vector
			len = FX_V3LEN( dir );
			FX_V3SCALE1( dir, 1/len );

			// get mouse offset vector
			FX_V3SET( plane, 0, 1, 0 );
			fxHandleMouseVector( hand, mouse, plane, 3, FX_NOFLAG );

			// find the direction (+/-) of the mouse movement (i.e. towards the target or away from it)
			dp = FX_V3DOT( mouse, dir );

			// get the magnitude of the mouse offset vector, this will be our scaling factor for our
			// normalized direction vector
			len = FX_V3LEN( mouse );
			
			// scale the direction vector and point it in the right direction
			FX_V3SCALE1( dir, len * (dp<0?-1:1) );

			// get local position of controller
			fxChannelsGet( ctlr, FX_POSITION, NOW, ctlrPos, FX_NOFLAG );

			// add offset vector to it
			FX_V3ADD1( ctlrPos, dir );

			// find out if we are in setup mode, if so: adjust setup values,
			// otherwise create/modify keys
			fxInterfaceMode( mode, 64, FX_NOFLAG );

			if( strcmp( mode, "Setup") != 0 )
			{
				// begin keyframe editing session
				fxKeyEditBegin( MoveTowards, ctlr, FX_NOFLAG );

					// create key if none exists
					keyCreated = fxKeyCreate( ctlr, NOW, &keyID, FX_NOFLAG );

					fxObjectGetName( ctlr, itemName, 64, FX_NOFLAG );
					fxMessageSend( FX_NULLID, "movetowards", FX_MESSAGE_INFO, itemName, NULL );

					// set the position at this key
					fxKeyChannelsSet( ctlr, keyID, FX_POSITION, ctlrPos, FX_NOFLAG );

					if( keyCreated == 1 )	// new key created
						fxKeyInclusionSet( ctlr, keyID, FX_POSITION_BITS, FX_NOFLAG );
					else if( keyCreated == 2 )	// key already existed
					{
						// get currently included channels
						fxKeyInclusionGet( ctlr, keyID, &keyInclusion, FX_NOFLAG );
						// if the position bits were not already set, set them now
						keyInclusion |= FX_POSITION_BITS;
						// set the included channels
						fxKeyInclusionSet( ctlr, keyID, keyInclusion, FX_NOFLAG );
					}
					// force the scene to re-calculate with the new changes
					fxFrameCalculate( NOW, FX_FCALC_UPDATE );
				
				// end keyframe editing session
				fxKeyEditEnd( MoveTowards, ctlr, FX_NOFLAG );
			}
			else
			{
				// If we're in Setup Mode, then set the Setup (rest) values of the controller
				fxChannelsSet( ctlr, FX_POSITION, ctlrPos, FX_CHANXFER_SETUP );
				// force the scene to re-calculate with the new changes
				fxFrameCalculate( NOW, FX_FCALC_UPDATE );
			}
			
			// update the interface
			fxInterfaceUpdate( FX_UPDATE_WORLD|FX_UPDATE_LIST|FX_UPDATE_GRAPH, FX_NOFLAG );

		}// ACT_PROCESS
		break;
	}

	return ENTRY_OK;

}
