/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

  Author: Lyle Milton
  Date: 06/27/04


  -----------------------------------------------------------------------------
  Very simple Image Filter shader.  Its only purpose is to process any connected
  shaders, effectively creating an overlay (or gel).  To accomplish this, it
  sets the shader_globals frame/pixel coordinate data.  It also sets _P & _Pl to
  that pixel coordinate to force any connected shaders to compute in image/screen
  space, rather than world or local.

=============================================================================*/
#include "globals.h"



//Shader Callbacks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
static FX_SHADERINITINPUTS(ShaderInitInputs);
static FX_SHADERFILTER(ShaderFilter);


//=======================================================================================
//MergeAccess
//		shader access callback
//=======================================================================================
FX_ACCESSFUNC(FilterAccess)
{
	FXshader		shader = (FXshader)ID;

	if (level==ACCESS_OBJECT)
	{
		switch(entry)
		{
			case O_CREATE:

				//#4. Create Inputs------------------------------------------------------------
				COLOR_IN		= fxShaderCreateInput(shader, "Color", FX_IOTYPE_COLOR, 0);

				//#5. Create Outputs-----------------------------------------------------------
				COLOR_OUT			= fxShaderCreateOutput(shader, "Color", FX_IOTYPE_COLOR, 0);

				//#6. Set Shader Callbacks-----------------------------------------------------
				fxShaderCallback(shader, FX_SHADERFUNC_INITINPUTS, &ShaderInitInputs);
				fxShaderCallback(shader, FX_SHADERFUNC_IMAGEFILTER, &ShaderFilter);

				break;

			default:
				break;
		}
	}
	else if (level==ACCESS_INTERFACE)
	{
		switch(entry)
		{
			case IN_CREATE:
				{
					//Assign the blocks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SHADER, FX_NULLID);
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SPLINE, FX_NULLID);
				}
				break;

			default:
				break;
		}
	}

	return ENTRY_OK;
}






//=======================================================================================
//ShaderInitInputs
//		Initialize the shader node inputs.
//=======================================================================================
static FX_SHADERINITINPUTS(ShaderInitInputs)
{

	return;
}


//=======================================================================================
//ShaderFilter
//		Evaluate the filter, filling in the buffer color data.
//=======================================================================================
static FX_SHADERFILTER(ShaderFilter)
{
	FX_BufferSample		*bfs = NULL;
	FX_ShaderGlobals	sg;
	FX_ShaderParam		color_value, alpha_value;
	FX_Color			color;
	int					x, y, x_pix, y_pix;

	//allocate a new buffer sample,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	if (!(bfs = fxBufferSampleCreate(0))) return;


	//traverse all of the pixels in this chunk, processing all connected input shaders.
	for (y=0;y<height;y++)
	{
		y_pix = posy + y;

		for (x=0;x<width;x++)
		{
			x_pix = posx + x;

			sg = *shader_globals; //preserve shader_globals

			//Get the pixel and fill in the frame and frame _P info,,,,,,,,,,,,
			if (!fxBufferPixel(shader_globals, x_pix, y_pix, bfs, FX_BUFFGLOBALS_FRAMEPOS|FX_BUFFGLOBALS_FRAME)) continue;

			//Get the buffer sample color,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			//NOTE: we could have easily gotten the color from the buffer and
			//automatically transfered it to shader_globals.  However, we're
			//just using the buffer sample for illustrative purposes.
			fxBufferSampleColor(bfs, &color, NULL, NULL, NULL, 0);

			//initialize the color and opacity,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			color_value.color = color;
			alpha_value.f = _in_opacity;

			//process any connected shaders to compute color and opacity,,,,,,,
			fxShaderProcessInput(shader, shader_globals, COLOR_IN, &color_value, 0);
			fxShaderProcessInput(shader, shader_globals, IN_OPACITY, &alpha_value, 0);

			//create the new color info from the computed color, to the original buffer color using the alpha
			FX_V3LERP(&color.r, &color.r, &color_value.color.r, alpha_value.f);

			//Set the color back to the buffer,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			fxBufferColorSet(shader_globals, x_pix, y_pix, color, 0);

			//restore shader_globals,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			*shader_globals = sg;
		}
	}

	//free the buffer sample,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	fxBufferSampleDelete(bfs, 0);

	return;
}

