/*=============================================================================
  Copyright (c) 1999-2003 pmG Worldwide, LLC.

  Author: Ron Griswold
  Date: 03/14/03

  Description: Simple effect that scales an object's points along their normals.
  See the tutorials section of the docs for a detailed description of this file.	

=============================================================================*/
#include "NormScale.h"

static FX_EFFECTSCAN(EffectScan, NormScaleState, state);
static FX_DISPLACESCAN(DisplaceScan, NormScaleState, state);

FX_ACCESSFUNC(NormScaleAccess)
// FXint f(FX_AccessInfo *ai, FXarb ID, FXint level, FXint64 entry)
{
	FXeffect effect = (FXeffect)ID;
	NormScaleState * state = NULL;

	switch( level )
	{

	case ACCESS_OBJECT:
		switch( entry )
		{
		case O_CREATE:	// Effect instance being created
			{	
				// Allocate memory for our state structure
				if( !(state = (NormScaleState *)calloc(1, sizeof(NormScaleState))) )
					return ENTRY_CONTINUE;

				// Attach the state data to the effect
				fxObjectSetTypeData( effect, state, FX_NOFLAG );

				// Set up channel
				fxChannelSetup( effect, "factor", FX_CHAN_0, FX_CHANTYPE_VALUE, state->scale, 0.0, 1.0, FX_NOFLAG);

				// Link channel 0 to state->scale
				fxChannelDataLink( effect, FX_CHAN_0, FX_ARG_DOUBLE, &state->scale, FX_NOFLAG);


			}// O_CREATE
			break;

		case O_DESTROY:	// Effect instance being destroyed
			{
				if( state = fxObjectGetTypeData( effect, FX_NOFLAG ) )
				{
					// Free memory
					free(state);

					// Set effect's state data as NULL
					fxObjectSetTypeData( effect, NULL, FX_NOFLAG);
				}
			}// O_DESTROY
			break;
		} // ACCESS_OBJECT
		break;

	case ACCESS_SYSTEM:
		switch( entry )
		{
		case S_MODULE_INIT:
			{
				// This effect will be represented by a slider in messiah's world view
				fxInitEffectManipulator(FX_MANIPULATOR_SLIDER, FX_NOFLAG);

				// Give the effect's icon a color
				fxInitEffectColor((FXubyte)0, (FXubyte)220, (FXubyte)200);

				// Set the type of tool used by the effect, a null in this case
				fxInitEffectTool(FX_TOOLTYPE_NULL, FX_NOFLAG);
				
			}// S_MODULE_INIT
			break;
		}// ACCESS_SYSTEM
		break;

	case ACCESS_PROCESS:
		switch( entry )
		{
		case P_POST_POINT_DISPLACE:
			{	
				if( state = fxObjectGetTypeData( effect, FX_NOFLAG ) )
				{
					// Process each target, EffectScan() is called once for each attached target
					fxEffectTargetScan( effect, &EffectScan, state, FX_NOFLAG );
				}
			}// P_POST_POINT_DISPLACE
			break;
		}// ACCESS_PROCESS
		break;

	}// switch level

	return ENTRY_OK;
}

static FX_EFFECTSCAN(EffectScan, NormScaleState, state)
//	FXint f(FXeffect effect, FXtool tool, FXobject target, dt *ed)
{
	// Process each point in target, DisplaceScan() gets called once for each point
	fxDisplaceScan( target, FX_NULLID, FX_NOFLAG, &DisplaceScan, state, FX_NOFLAG );

	return FX_TRUE;
}

static FX_DISPLACESCAN(DisplaceScan, NormScaleState, state)
//	FXint f(FX_DisplacePoint *p, dt *ed)
{
	FXdouble norm[3];

	// Get surface normal at point p->ID of object p->objID and store it in norm
	fxMeshPointNormal( p->objID, p->ID, norm, FX_NOFLAG );

	// Calculate displacement along normal and set the point's new position (p->new_pos)
	// Note: positions are in world space
	p->new_pos[0] = p->cur_pos[0] + (state->scale * norm[0]);
	p->new_pos[1] = p->cur_pos[1] + (state->scale * norm[1]);
	p->new_pos[2] = p->cur_pos[2] + (state->scale * norm[2]);

	return FX_TRUE;
}
