/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_effect.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 05/09/04 - LM

	-----------------------------------------------------------------
	EFFECT SETUP
	-----------------------------------------------------------------
	If you need to create an effect module that does motion modification
	and/or mesh displacements, you'll find	that messiah provides very
	easy setup & access, and allows for great flexibility.  All that
	is required in setting up a mesh displacement effect is requesting
	the proper access.

	EX1: Requesting Displacement & Motion Access

		_pluginEntry
		{
			-
			-
			-
			fxModuleAccess(ACCESS_PROCESS, P_POST_POINT_DISPLACE|P_POST_MOTION, 0);
			fxModuleRegister(FX_MODTYPE_EFFECT, "MyEffectModule", &EffectAccess, 0);
			-
			-
			-

	In EX1, setting the desired process access in the ACCESS_PROCESS
	level is all that's necessary to tell messiah that your module
	should be called for effect processing.  The next step is just
	defining your processing code.

	See messiah_access.h for further details on requesting access.

	Note that EX1 is setting both DISPLACE and MOTION.  This means the
	effect module will receive both messages.  This is unique amoung
	API's in that you are not required to define a module as a 
	displacer OR motion module.  While it's not common for effects to
	need to do both, in messiah your module is free to perform
	displacement AND motion as your needs dictated.  In most case,
	however, you'd either use DISPLACE or MOTION.


	-----------------------------------------------------------------
	In messiah, a single effect can modify multiple targets.  This means
	that effect modules must be prepared to process multiple targets.
	messiah provides a callback mechanism that makes this easy.  First
	you need to define your target scanning callback.

	EX2: Defining A Target Scanning Callback

		FXintf EffectScan(FXeffect effect, FXtool tool, FXobject target, My_EffectData *data)

		-OR-

		FX_EFFECTSCAN(EffectScan, My_EffectData, data)

		{
			//todo: diplace the target
			-
			-
			-
			return FX_TRUE;
		}


	Once your target scanning function is defined, you can then respond
	to the displace message in your Access callback.


	EX3: Defining A Target Scanning Callback

		case P_POST_POINT_DISPLACE:
			-
			-
			-
			fxEffectTargetScan(effect, &EffectScan, data, 0);


	The fxEffectTargetScan function will traverse all of the target
	objects for your effect.  Your FX_EFFECTSCAN function will be
	called for each target, and you will receive the target & tool in
	order to perform your effect's displacement operations.

	See messiah_mesh.h for information on mesh displacement.


=============================================================================*/

#ifdef _MESSIAH_API_EFFECT

#ifndef __MESSIAH_EFFECT
#define __MESSIAH_EFFECT

#ifndef __MESSIAH_HIDE_TYPES

	#define FX_EFFECTSCAN(f, dt, ed)	FXintf f(FXeffect effect, FXtool tool, FXobject target, dt *ed)

#endif //__MESSIAH_HIDE_TYPES



FX_API(FXeffect, fxEffectCreate)(FXchar *effect_type, FXchar *effect_name, FXobject target, FXint flags);

FX_API(FXint, fxEffectTargetScan)(FXeffect, FXint (*scan_func)(FXeffect, FXtool, FXobject, FXvoid*), FXvoid *data, FXint flags);

FX_API(FXtool, fxEffectTool)(FXeffect, FXint flags);

FX_API(FXobject, fxEffectTargetFirst)(FXeffect, FXint flags);

FX_API(FXobject, fxEffectTargetNext)(FXeffect, FXobject currentID, FXint flags);

FX_API(FXint, fxInitEffectManipulator)(FXchar *manipulator, FXint flags);

FX_API(FXint, fxInitEffectColor)(FXubyte red, FXubyte green, FXubyte blue);

FX_API(FXint, fxInitEffectTool)(FXchar *tool_name, FXint flags);

FX_API(FXobject, fxEffectFirst)(FXint flags);

FX_API(FXobject, fxEffectNext)(FXeffect currentID, FXint flags);


#define _MESSIAH_PTR_EFFECT	messiah_apicomponent = FX_APIFUNCS_EFFECT;\
							FX_PTR(fxEffectCreate);\
							FX_PTR(fxEffectTargetScan);\
							FX_PTR(fxEffectTool);\
							FX_PTR(fxEffectTargetFirst);\
							FX_PTR(fxEffectTargetNext);\
							FX_PTR(fxInitEffectManipulator);\
							FX_PTR(fxInitEffectColor);\
							FX_PTR(fxInitEffectTool);\
							FX_PTR(fxEffectFirst);\
							FX_PTR(fxEffectNext)

#endif //__MESSIAH_EFFECT

#endif //_MESSIAH_API_EFFECT
