/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_getapifunc.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 04/25/04 - LM
			-added WEIGHTTOOL & LIGHT

	
	------------------------------------------------------------

	This function is used to get the messiah API component functions.

	EX:
		pluginGetAPIComponents(FX_APIFUNCS_OBJECT|FX_APIFUNCS_GEOMETRY);

	In that example, the API functions for Object & Geometry
	manipulation are retrieved.  Note, however, that even through
	you've requested a specific API component, That doesn't always
	mean you can just start calling the functions.  Certain API 
	functions should only be used at certain entry points/messages.
	For example, FX_APIFUNCS_INTERFACE control creation functions
	should only be used in your ACCESS_INTERFACE->IN_CREATE message.

	EX:

		FX_ACCESSFUNC(ModuleAccess)
		{
			switch(level)
			{
				case ACCESS_INTERFACE:
					switch(entry)
						-
						-
						-
						case IN_CREATE:
							mycon.block = fxCC_Block("Hello World", &HelloWorldBlock, ACC_DRAW|ACC_SIZE, 0);
							mycon.button = fxCC_Button("Hi!", &HelloWorldButton, ACC_VALUE, 100);
							-
							-
							-

=============================================================================*/


#ifndef	__MESSIAH_GETAPICOMPONENTS
#define __MESSIAH_GETAPICOMPONENTS


#define GET_COMP(cmp)	if ((type_flag&FX_APIFUNCS_##cmp)&&(!(messiah_apicomponents_retrieved&FX_APIFUNCS_##cmp)))\
						{\
							messiah_apicomponents_retrieved |= FX_APIFUNCS_##cmp;\
							_MESSIAH_PTR_##cmp;\
						}

#define GET_TOOLCOMP(cmp)	if (type_flag&FX_APIFUNCS_TOOLTYPE)\
							{\
								_MESSIAH_PTR_##cmp;\
							}


//pluginGetAPIComponents Declaration/Definition,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
FX_FUNC(FXint,	pluginGetAPIComponents)(FXint type_flag)
#ifndef _MAINPLUGIN_FILE
;
#else
{
	#ifdef _MESSIAH_PTR_MESSAGE

		GET_COMP(MESSAGE);

	#endif //_MESSIAH_PTR_MESSAGE

	#ifdef _MESSIAH_PTR_OBJECT

		GET_COMP(OBJECT);

	#endif //_MESSIAH_PTR_OBJECT

	#ifdef _MESSIAH_PTR_MESH

		GET_COMP(MESH);

	#endif //_MESSIAH_PTR_MESH

	#ifdef _MESSIAH_PTR_MOTION

		GET_COMP(MOTION);

	#endif //_MESSIAH_PTR_MOTION

	#ifdef _MESSIAH_PTR_INTERFACE

		GET_COMP(INTERFACE);

	#endif //_MESSIAH_PTR_INTERFACE

	#ifdef _MESSIAH_PTR_EFFECT

		GET_COMP(EFFECT);

	#endif //_MESSIAH_PTR_EFFECT

	#ifdef _MESSIAH_PTR_SHADER

		GET_COMP(SHADER);

	#endif //_MESSIAH_PTR_SHADER

	#ifdef _MESSIAH_PTR_IMAGE

		GET_COMP(IMAGE);

	#endif //_MESSIAH_PTR_IMAGE

	#ifdef _MESSIAH_PTR_FUNCTION

		GET_COMP(FUNCTION);

	#endif //_MESSIAH_PTR_FUNCTION

	#ifdef _MESSIAH_PTR_COMMAND

		GET_COMP(COMMAND);

	#endif //_MESSIAH_PTR_COMMAND

	#ifdef _MESSIAH_PTR_DATA

		GET_COMP(DATA);

	#endif //_MESSIAH_PTR_DATA

	#ifdef _MESSIAH_PTR_ARG

		GET_COMP(ARG);

	#endif //_MESSIAH_PTR_ARG

	#ifdef _MESSIAH_PTR_FILE

		GET_COMP(FILE);

	#endif //_MESSIAH_PTR_FILE

	#ifdef _MESSIAH_PTR_STRING

		GET_COMP(STRING);

	#endif //_MESSIAH_PTR_STRING

	#ifdef _MESSIAH_PTR_TIME

		GET_COMP(TIME);

	#endif //_MESSIAH_PTR_TIME

	#ifdef _MESSIAH_PTR_MODULE

		GET_COMP(MODULE);

	#endif //_MESSIAH_PTR_MODULE

	#ifdef _MESSIAH_PTR_EXPFUNC

		GET_COMP(EXPFUNC);

	#endif //_MESSIAH_PTR_EXPFUNC

	#ifdef	_MESSIAH_PTR_VARIABLE

		GET_COMP(VARIABLE);

	#endif

	#ifdef	_MESSIAH_PTR_GROUP

		GET_COMP(GROUP);

	#endif

	#ifdef	_MESSIAH_PTR_ARMATURE

		GET_COMP(ARMATURE);

	#endif

	#ifdef	_MESSIAH_PTR_ACTION

		GET_COMP(ACTION);

	#endif

	#ifdef _MESSIAH_PTR_UTILITY

		GET_COMP(UTILITY);

	#endif

	#ifdef	_MESSIAH_PTR_WEIGHTTOOL

		GET_TOOLCOMP(WEIGHTTOOL);
			
	#endif

	#ifdef	_MESSIAH_PTR_SKELETON

		GET_TOOLCOMP(SKELETON);

	#endif

	#ifdef	_MESSIAH_PTR_LIGHT

		GET_TOOLCOMP(LIGHT);

	#endif

	return FX_TRUE;
}
#endif //_MAINPLUGIN_FILE
//==================================================================================


#endif //__MESSIAH_GETAPICOMPONENTS
