/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_module.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 02/15/05 - LM
			- added FX_MODEEFFECT and FX_MODACTION flags

	-----------------------------------------------------------------
	WEIGHT TOOLS
	-----------------------------------------------------------------
	Normally, tools are used to allow certain effects to perform their
	basic function.  For example, the Curve tool is used with Flex to
	bend objects.  Some tools surve an additional purpose: providing
	weight data for targets for general use throught the system.  
	
	The API will allow you to create tools, but in a limited fashion.
	For example, it's not possible to create tools as complex as
	MetaEffector, Skeleton, or Curve at this time.  However, you can
	create basic tools to compute weight data.  This can be used to
	write modules that import weight data from other sources, or
	compute custom weight values.

	See messiah_weighttool.h for more detailed info on creating Weight Tools.

=============================================================================*/


#ifdef _MESSIAH_API_MODULE

#ifndef	__MESSIAH_MODULE
#define __MESSIAH_MODULE


#ifndef	__MESSIAH_HIDE_TYPES

#endif	//__MESSIAH_HIDE_TYPES


#define FX_MODTYPE_EFFECT			1 //Effect module
#define FX_MODTYPE_SHADER			2 //Shader module
#define FX_MODTYPE_TOOL				3 //Tool module
#define FX_MODTYPE_EDIT				4 //Edit extension module
#define FX_MODTYPE_PLAY				5 //Play extension module
#define FX_MODTYPE_CUSTOMIZE		6 //Customize extension module
#define FX_MODTYPE_SETUP			13 //Setup extension module
#define FX_MODTYPE_ACTION			14 //Armature Action module



#define FX_MODFLAG_EFFECTS			(1<<FX_MODTYPE_EFFECT) //Effect module
#define FX_MODFLAG_SHADERS			(1<<FX_MODTYPE_SHADER) //Shader module
#define FX_MODFLAG_TOOLS			(1<<FX_MODTYPE_TOOL) //Tool module
#define FX_MODFLAG_EDITS			(1<<FX_MODTYPE_EDIT) //Edit extension module
#define FX_MODFLAG_PLAYS			(1<<FX_MODTYPE_PLAY) //Play extension module
#define FX_MODFLAG_CUSTOMIZES		(1<<FX_MODTYPE_CUSTOMIZE) //Customize extension module
#define FX_MODFLAG_FUNCTIONS		(1<<10) //Function
#define FX_MODFLAG_COMMANDS			(1<<11) //Command
#define FX_MODFLAG_SETUPS			(1<<FX_MODTYPE_SETUP) //Setup extension modules
#define FX_MODFLAG_ACTIONS			(1<<FX_MODTYPE_ACTION) //Action modules


//Flags for use with fxModuleRegister,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define FX_MODEFFECT_START(n)		(n)
#define _MODEFFECT_BASE(f)			(1<<(f))
#define FX_MODEFFECT_OVERRIDEPROC	_MODEFFECT_BASE(0) //effect will handle all process messages (user can't set the process order)
#define FX_MODEFFECT_MOTION			_MODEFFECT_BASE(1) //explicitly notifies the system that the effect modifies motion (normally this is assumed via the proc access)
#define FX_MODEFFECT_DISPLACEMENT	_MODEFFECT_BASE(2) //explicitly notifies the system that the effect modifies points (normally this is assumed via the proc access)

#define _MODSHADER_BASE(f)			(1<<(6+(f)))
#define FX_MODSHADER_IMAGEFILTER	_MODSHADER_BASE(0) //shader is an image filter

#define _MODTOOL_BASE(f)			(1<<(12+(f)))
#define FX_MODTOOL_WEIGHT			_MODTOOL_BASE(0) //tool can be used as a weight

#define _MODACTION_BASE(f)			(1<<(20+(f)))
#define FX_MODACTION_MOUSEPROC		_MODACTION_BASE(0) //action will process all mouse messages (user can't set the mouse action type)
#define FX_MODACTION_COORDSYSOPS	_MODACTION_BASE(1) //action will display channel and coodinate system options on button items




FX_API(FXint, fxModuleDescription)(FXchar *desc, FXint flags);

FX_API(FXint, fxModuleAccess)(FXint level, FXint64 entry, FXint clear);

FX_API(FXmodule, fxModuleRegister)(FXint type, FXchar *name, FXvoid *access_func, FXint flags);

FX_API(FXint, fxModuleArgInt)(FXchar *name, FXchar *abbr, FXint def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgDouble)(FXchar *name, FXchar *abbr, FXdouble def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgFloat)(FXchar *name, FXchar *abbr, FXfloat def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgInt3)(FXchar *name, FXchar *abbr, FXint def1, FXint def2, FXint def3, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgDouble3)(FXchar *name, FXchar *abbr, FXdouble def1, FXdouble def2, FXdouble def3, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgFloat3)(FXchar *name, FXchar *abbr, FXfloat def1, FXfloat def2, FXfloat def3, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgInt4)(FXchar *name, FXchar *abbr, FXint def1, FXint def2, FXint def3, FXint def4, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgDouble4)(FXchar *name, FXchar *abbr, FXdouble def1, FXdouble def2, FXdouble def3, FXdouble def4, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgFloat4)(FXchar *name, FXchar *abbr, FXfloat def1, FXfloat def2, FXfloat def3, FXfloat def4, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgChar)(FXchar *name, FXchar *abbr, FXchar def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgString)(FXchar *name, FXchar *abbr, FXchar *def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgStringList)(FXchar *name, FXchar *abbr, FXchar **def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgVoid)(FXchar *name, FXchar *abbr, FXvoid *def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgDP)(FXchar *name, FXchar *abbr, FXvoid **def, FXchar *def_exp, FXint flags);

FX_API(FXint, fxModuleArgObj)(FXchar *name, FXchar *abbr, FXobject def, FXchar *def_exp, FXint flags);



#define _MESSIAH_PTR_MODULE	messiah_apicomponent = FX_APIFUNCS_MODULE;\
							FX_PTR(fxModuleDescription);\
							FX_PTR(fxModuleAccess);\
							FX_PTR(fxModuleRegister);\
							\
							FX_PTR(fxModuleArgInt);\
							FX_PTR(fxModuleArgDouble);\
							FX_PTR(fxModuleArgFloat);\
							FX_PTR(fxModuleArgInt3);\
							FX_PTR(fxModuleArgDouble3);\
							FX_PTR(fxModuleArgFloat3);\
							FX_PTR(fxModuleArgInt4);\
							FX_PTR(fxModuleArgDouble4);\
							FX_PTR(fxModuleArgFloat4);\
							FX_PTR(fxModuleArgChar);\
							FX_PTR(fxModuleArgString);\
							FX_PTR(fxModuleArgStringList);\
							FX_PTR(fxModuleArgVoid);\
							FX_PTR(fxModuleArgDP);\
							FX_PTR(fxModuleArgObj)

#endif	//__MESSIAH_MODULE

#endif	//_MESSIAH_API_MODULE
